<?php
/**
 * Cos For Typecho,可直接进入后台面板设置
 *
 * @package cosUpload
 * @author 菜菜子
 * @version 2.0
 * @link https://www.cc430.cn/index.php/archives/129/
 * @date 2017.1.8
 */
require('include/connect.php');
use qcloudcos\Cosapi;

class cosUpload_Plugin implements Typecho_Plugin_Interface
{
    /**
     * 激活插件方法,如果激活失败,直接抛出异常
     *
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function activate()
    {
        //上传
        Typecho_Plugin::factory('Widget_Upload')->uploadHandle = array('cosUpload_Plugin', 'uploadHandle');
        //修改
        Typecho_Plugin::factory('Widget_Upload')->modifyHandle = array('cosUpload_Plugin', 'modifyHandle');
        //删除
        Typecho_Plugin::factory('Widget_Upload')->deleteHandle = array('cosUpload_Plugin', 'deleteHandle');
        //路径参数处理
        Typecho_Plugin::factory('Widget_Upload')->attachmentHandle = array('cosUpload_Plugin', 'attachmentHandle');
        //文件内容数据
        Typecho_Plugin::factory('Widget_Upload')->attachmentDataHandle = array('cosUpload_Plugin', 'attachmentDataHandle');
    }

    /**
     * 禁用插件方法,如果禁用失败,直接抛出异常
     *
     * @static
     * @access public
     * @return void
     * @throws Typecho_Plugin_Exception
     */
    public static function deactivate()
    {
    }

    /**
     * 获取插件配置面板
     *
     * @access public
     * @param Typecho_Widget_Helper_Form $form 配置面板
     * @return void
     */
    public static function config(Typecho_Widget_Helper_Form $form)
    {
        $appid = new Typecho_Widget_Helper_Form_Element_Text('appid',
            null, '',
            _t('APPID：'),
            _t('<a href="https://console.qcloud.com/cos4/secret" target="_blank">获取APPID</a>'));
        //$form->addInput($appid->addRule('required', _t('APPID 不能为空！')));
        $form->addInput($appid);
        $ak = new Typecho_Widget_Helper_Form_Element_Text('ak',
            NULL, '',
            _t('SecretId：'),
            _t('<a href="https://console.qcloud.com/cos4/secret" target="_blank">获取SecretId</a>'));
        $form->addInput($ak);

        $sk = new Typecho_Widget_Helper_Form_Element_Text('sk',
            NULL, '',
            _t('SecretKey：'),
            _t('<a href="https://console.qcloud.com/cos4/secret" target="_blank">获取SecretKey</a>'));
        $form->addInput($sk);


        $region = new Typecho_Widget_Helper_Form_Element_Radio('region',
            array('gz' => _t('华南'), 'sh' => _t('华中'), 'tj' => _t('华北')),
            'gz',
            _t('选择bucket节点')
        );
        $form->addInput($region);


        $bucketName = new Typecho_Widget_Helper_Form_Element_Text('bucket',
            NULL, 'bucketName',
            _t('Bucket名称：'),
            _t('例如test'));
        $form->addInput($bucketName->addRule('required', _t('Bucket名称不能为空！')));


        $domain = new Typecho_Widget_Helper_Form_Element_Text('domain',
            NULL, 'http://',
            _t('使用的域名,必填,请带上http://或https://，可使用默认域名或自定义域名'),
            _t('在bucket中的域名管理<br>默认域名形如：http://test-125122576.cosgz.myqcloud.com<br>自定义域名形如：https://cos.cc430.cn'));
        $form->addInput($domain->addRule('required', _t('cos域名不能为空！')));;
    }

    /**
     * 个人用户的配置面板
     *
     * @access public
     * @param Typecho_Widget_Helper_Form $form
     * @return void
     */
    public static function personalConfig(Typecho_Widget_Helper_Form $form)
    {
    }

    /**
     * 上传文件处理函数
     *
     * @access public
     * @param array $file 上传的文件
     * @return mixed
     */
    public static function uploadHandle($file)
    {
        if (empty($file['name'])) return false;
        //获取扩展名
        $ext = self::getSafeName($file['name']);
        //判定是否是允许的文件类型
        if (!Widget_Upload::checkFileType($ext)) return false;
        //获取文件名
        $filePath = '/' . date('Y') . '/' . date('m') . '/' . date('d') . '/';
        $fileName = sprintf('%u', crc32(uniqid())) . '.' . $ext;
        //如果没有临时文件，则退出
        if (!isset($file['tmp_name'])) return false;
        //获取插件参数
        $options = Typecho_Widget::widget('Widget_Options')->plugin('cosUpload');
        //初始化cos

        \qcloudcos\Conf::__cosinit__($options);
        \qcloudcos\Cosapi:: region($options);
        $ispath = Cosapi::statFolder($options->bucket, $filePath);
        if ($ispath['code'] != '0') {
            //没有目录先创建
            Cosapi::createFolder($options->bucket, $filePath);
        }

        $dstPath = $filePath . $fileName;

        $srcPath = $file['tmp_name'];

        Cosapi::upload($options->bucket, $srcPath, $dstPath);

        return array(
            'name' => $file['name'],
            'path' => $filePath . $fileName,
            'size' => $file['size'],
            'type' => $ext,
            'mime' => @Typecho_Common::mimeContentType(rtrim($options->domain, '/') . '/' . $filePath . $fileName),
        );
    }

    /*
     * 初始化bucket位置
     */
    public static function region($option){
        $region=$option->region;
        return $region;
    }

    /**
     * 文件修改处理函数
     *
     * @access public
     * @param array $content 当前文件信息
     * @param array $file 新上传文件信息
     * @return mixed
     */
//    public static function modifyHandle($content, $file)
//    {
//        if (empty($file['name'])) return false;
//        //获取扩展名
//        $ext = self::getSafeName($file['name']);
//        if ($content['attachment']->type != $ext) return false;
//        //获取文件名
//        $fileName = $content['attachment']->path;
//        //获取插件参数
//        $options = Typecho_Widget::widget('Widget_Options')->plugin('cosUpload');
//        $bucket = $options->bucket;
//        //初始化
//        self::__cosinit__($options);
//        //如果没有临时文件，则退出
//        if (!isset($file['tmp_name'])) return false;
//        //重新上传
//
//        list($ret, $err) = Qiniu_PutFile($upToken, $fileName, $file['tmp_name'], $putExtra);
//        if ($err !== null) return false;
//        return array(
//            'name' => $file['name'],
//            'path' => $fileName,
//            'size' => $file['size'],
//            'type' => $ext,
//            'mime' => @Typecho_Common::mimeContentType(rtrim($options->domain, '/') . '/' . $fileName),
//        );
//    }

    /**
     * 文件删除
     *
     * @access public
     * @param array $content 当前文件信息
     * @return mixed
     */
    public static function deleteHandle($content)
    {
        $options = Typecho_Widget::widget('Widget_Options')->plugin('cosUpload');
        \qcloudcos\Conf::__cosinit__($options);
        \qcloudcos\Cosapi:: region($options);
        $err = Cosapi::delFile($options->bucket, $content['attachment']->path);
        return !$err;
    }

    /**
     * 获取实际文件数据
     *
     * @access public
     * @param array $content
     * @return string
     */
    public static function attachmentDataHandle($content)
    {
        $options = Typecho_Widget::widget('Widget_Options')->plugin('cosUpload');
        \qcloudcos\Conf::__cosinit__($options);
        \qcloudcos\Cosapi:: region($options);
        list($ret, $err) = Cosapi::stat($options->bucket, $content['attachment']);
        return $err === null ? $ret : false;
    }

    /**
     * 获取实际文件绝对访问路径
     *
     * @access public
     * @param array $content 文件相关信息
     * @return string
     */
    public static function attachmentHandle(array $content)
    {
        $domain = Typecho_Widget::widget('Widget_Options')->plugin('cosUpload')->domain;
        $tmp = preg_match('/http(s)?:\/\/[\w\d\.\-\/]+$/is', $domain);    //粗略验证域名
        if (!$tmp) return false;
        return Typecho_Common::url($content['attachment']->path, $domain);
    }


    /**
     * 获取安全的文件名
     *
     * @param string $name
     * @static
     * @access private
     * @return string
     */
    private static function getSafeName(&$name)
    {
        $name = str_replace(array('"', '<', '>'), '', $name);
        $name = str_replace('\\', '/', $name);
        $name = false === strpos($name, '/') ? ('a' . $name) : str_replace('/', '/a', $name);
        $info = pathinfo($name);
        $name = substr($info['basename'], 1);
        return isset($info['extension']) ? strtolower($info['extension']) : '';
    }
}
